// Crc32.cpp : implementation file
//

#include "stdafx.h"

#include "Crc32.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCrc32
//------------------------------------------------------
//     High Performance CRC-32 Computation Routine      
//                                                      
//        Copyright 1988 W. David Schwaderer            
//             All rights reserved                      
//                                                      
//       Warning...this program uses bit fields!        
//        For warnings on bit field hazards see:        
//                                                      
//        The C Wizard's Programming Reference          
//                W. David Schwaderer                   
//                 Wiley Press, 1985                    
//                                                      
//------------------------------------------------------

CCrc32::CCrc32()
{
    GenerateTable();
}

CCrc32::~CCrc32()
{
}

void CCrc32::GenerateTable()   // generate the look-up table 
{
    bit16 word;
    bit32 dword;

	for ( word.w = 0; word.w < 256; word.w++ )  
	{
		dword.dw = 0;      // zero out the value

		dword.bit.b31 = word.bit.b1 ^ word.bit.b7;
		dword.bit.b30 = word.bit.b0 ^ word.bit.b1 ^ word.bit.b6 ^ word.bit.b7;
		dword.bit.b29 = word.bit.b0 ^ word.bit.b1 ^ word.bit.b5 ^ word.bit.b6 ^ word.bit.b7;
		dword.bit.b28 = word.bit.b0 ^ word.bit.b4 ^ word.bit.b5 ^ word.bit.b6;
		dword.bit.b27 = word.bit.b1 ^ word.bit.b3 ^ word.bit.b4 ^ word.bit.b5 ^ word.bit.b7;
		dword.bit.b26 = word.bit.b0 ^ word.bit.b1 ^ word.bit.b2 ^ word.bit.b3 ^ word.bit.b4 ^ word.bit.b6 ^ word.bit.b7;
		dword.bit.b25 = word.bit.b0 ^ word.bit.b1 ^ word.bit.b2 ^ word.bit.b3 ^ word.bit.b5 ^ word.bit.b6;
		dword.bit.b24 = word.bit.b0 ^ word.bit.b2 ^ word.bit.b4 ^ word.bit.b5 ^ word.bit.b7;
		dword.bit.b23 = word.bit.b3 ^ word.bit.b4 ^ word.bit.b6 ^ word.bit.b7;
		dword.bit.b22 = word.bit.b2 ^ word.bit.b3 ^ word.bit.b5 ^ word.bit.b6;
		dword.bit.b21 = word.bit.b2 ^ word.bit.b4 ^ word.bit.b5 ^ word.bit.b7;
		dword.bit.b20 = word.bit.b3 ^ word.bit.b4 ^ word.bit.b6 ^ word.bit.b7;
		dword.bit.b19 = word.bit.b1 ^ word.bit.b2 ^ word.bit.b3 ^ word.bit.b5 ^ word.bit.b6 ^ word.bit.b7;
		dword.bit.b18 = word.bit.b0 ^ word.bit.b1 ^ word.bit.b2 ^ word.bit.b4 ^ word.bit.b5 ^ word.bit.b6;
		dword.bit.b17 = word.bit.b0 ^ word.bit.b1 ^ word.bit.b3 ^ word.bit.b4 ^ word.bit.b5;
		dword.bit.b16 = word.bit.b0 ^ word.bit.b2 ^ word.bit.b3 ^ word.bit.b4;
		dword.bit.b15 = word.bit.b2 ^ word.bit.b3 ^ word.bit.b7;
		dword.bit.b14 = word.bit.b1 ^ word.bit.b2 ^ word.bit.b6;
		dword.bit.b13 = word.bit.b0 ^ word.bit.b1 ^ word.bit.b5;
		dword.bit.b12 = word.bit.b0 ^ word.bit.b4;
		dword.bit.b11 = word.bit.b3;
		dword.bit.b10 = word.bit.b2;
		dword.bit.b9  = word.bit.b7;
		dword.bit.b8  = word.bit.b1 ^ word.bit.b6 ^ word.bit.b7;
		dword.bit.b7  = word.bit.b0 ^ word.bit.b5 ^ word.bit.b6;
		dword.bit.b6  = word.bit.b4 ^ word.bit.b5;
		dword.bit.b5  = word.bit.b1 ^ word.bit.b3 ^ word.bit.b4 ^ word.bit.b7;
		dword.bit.b4  = word.bit.b0 ^ word.bit.b2 ^ word.bit.b3 ^ word.bit.b6;
		dword.bit.b3  = word.bit.b1 ^ word.bit.b2 ^ word.bit.b5;
		dword.bit.b2  = word.bit.b0 ^ word.bit.b1 ^ word.bit.b4;
		dword.bit.b1  = word.bit.b0 ^ word.bit.b3;
		dword.bit.b0  = word.bit.b2;

		m_crcTable[word.w] = dword.dw;
  }
}

/////////////////////////////////////////////////////////////////////////////
// CCrc32 member functions

DWORD CCrc32::GetCRC( const unsigned char * pTextPtr, int nLength, DWORD dwInitialCrc ) const
{
	int i, nIndex;
	DWORD dwCrc;

	dwCrc = dwInitialCrc ^ 0xffffffff;      // CRC starts as all Fs for each message

	for ( i = 0; i < nLength; i++, pTextPtr++ ) 
	{
		nIndex = (dwCrc ^ *pTextPtr) & 0x000000FFL;
		dwCrc = ((dwCrc >> 8) & 0x00FFFFFF) ^ m_crcTable[nIndex];
    }

    return dwCrc ^ 0xffffffff;	  
}
