#pragma once

// zipfile format: 

// [local file header 1]
// [file data 1]
// [data_descriptor 1 (optional)]
// [local file header 2]
// [file data 2]
// [data_descriptor 2 (optional)]
// ...
// [central file header 1] 
// [central file header 2] 
// ...
// [central directory end record]

const int LOCALFILE_HEADER_SIGNATURE = 0x04034b50;
const int CENTRALDIR_HEADER_SIGNATURE =  0x02014b50;
const int CENTRALDIR_INFO_SIGNATURE = 0x06054b50;
const int CURRENT_PKZIP_VERSION = 2;

const int MAX_ZIP_COMMENT_LENGTH = 0xffff;

#define MAKEVERSIONNUMBER(major,minor) ((BYTE)(((major)*10) + ((minor)&0x0f)))

#pragma pack(1)

typedef struct tagVersionInfo {
   BYTE cNumber;     // MAKEVERSIONNUMBER
   BYTE cHostOS;
} VERSIONINFO;

typedef enum { deflateNormal, deflateMaximum, deflateFast, deflateSuperfast } enumDeflatingMethod;

typedef struct {
    WORD encryptedFile : 1;
    WORD deflateMethod : 2;
    WORD dataDescriptorUsed : 1;
    WORD reserved : 3;
    WORD unused : 9;
} GENERALFLAG;

typedef enum  {    stored, 
                   shrunk, 
                   reduced1, 
                   reduced2, 
                   reduced3,
                   reduced4,
                   imploded,
                   _reserved_,
                   deflated
} enumCompressionMethod;

typedef struct {
	WORD twoSeconds : 5;		// 0 - 29
	WORD minute : 6;			// 0 - 59
	WORD hour : 5;				// 0 - 23
} DOSTIME;

typedef struct {
	WORD day : 5;				// 1 - 31
	WORD month : 4;				// 1 - 12
	WORD year : 7;				// +1980
} DOSDATE;

typedef struct {
	DWORD dwSignature;               		// local file header signature (0x04034b50)
	VERSIONINFO versionNeededToExtract;    // version needed to extract  
	GENERALFLAG flags;                     	// general purpose bit flag   
	WORD wCompressionMethod;                // compression method         
	DOSTIME fileTime;						// last mod file time (standard DOS format)
	DOSDATE fileDate;						// last mod file date         
	DWORD dwCrc32;                   		// crc-32                          
	DWORD dwCompressedSize;          		// compressed size                 
	DWORD dwUncompressedSize;        		// uncompressed size
	WORD wFilenameLength;               	// filename length
	WORD wExtraFieldLength;             	// extra field length
	// filename (variable size)
	// extra field (variable size)
} LOCALFILEHEADER;

const  int OS_FAT = 0;

typedef struct {
	DWORD dwSignature;               		// 4 bytes  (0x02014b50)
	VERSIONINFO versionMadeBy;          	//2 bytes
	VERSIONINFO versionNeededToExtract;    // needed to extract       2 bytes
	GENERALFLAG flags;                     	// general purpose bit flag        2 bytes
	WORD wCompressionMethod;                // compression method              2 bytes
	DOSTIME fileTime;						// last mod file time (standard DOS format)
	DOSDATE fileDate;						// last mod file date         
	DWORD dwCrc32;                   		// crc-32                          4 bytes
	DWORD dwCompressedSize;          		// compressed size                 4 bytes
	DWORD dwUncompressedSize;        		// 4 bytes
	WORD wFilenameLength;               	// 2 bytes
	WORD wExtraFieldLength;             	// 2 bytes
	WORD wFileCommentLength;            // 2 bytes
	WORD wLocatedOnDiskNo;           // 2 bytes, always 1 for one zip archive
	WORD wInternalFileAttributes;          // 2 bytes
	DWORD dwExternalFileAttributes;        // 4 bytes
	DWORD dwLocalHeaderOffset;          // relative offset of local header 4 bytes
} CENTRALFILEHEADER;
//	filename (variable size)
//	extra field (variable size)
//	file comment (variable size)

typedef struct {
		DWORD dwSignature;			            // end of central dir signature (0x06054b50)
        WORD wThisDiskNumber;    	            // number of this disk (1)
        WORD wCentralDirFirstDisk;	            // number of the disk with the start of the central directory
        WORD wThisDiskCentralDirEntriesCount;   // total number of entries in the central dir on this disk
        WORD wTotalCentralDirEntriesCount;      // total number of entries in the central dir
        DWORD dwCentralDirSize;		            // size of the central directory
        DWORD dwCentralDirOffset;	            // offset of start of central directory with respect to the starting disk number
        WORD wZipFileCommentLength;	            // zipfile comment length
} CENTRALDIRECTORYINFO;
// zipfile comment (variable size) 

#pragma pack()

