//////////////////////////////////////
// Vladimir Klisic (klisa@sezampro.yu)
// PC #45, April 1999.

#include "stdafx.h"
#include "zlib.h"
#include "zutil.h"

#include "Pkzip.h"
#include "ZipArchive.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CZipFile

CZipFile::CZipFile( const CENTRALFILEHEADER& cfh, const CString& strFilename, LPCTSTR pszFileComment )
    : m_bAttributes((BYTE)cfh.dwExternalFileAttributes),
    m_fileTime(cfh.fileTime), 
    m_fileDate(cfh.fileDate),
    m_compressionMethod( (enumCompressionMethod) cfh.wCompressionMethod ),
	m_dwCrc32(cfh.dwCrc32),
	m_dwCompressedSize(cfh.dwCompressedSize),
	m_dwUncompressedSize(cfh.dwUncompressedSize),
	m_dwFileOffset(cfh.dwLocalHeaderOffset),
    m_strFileComment(pszFileComment)
{
    m_strFilename = GetZipCompatibleFilename( strFilename );
}

CZipFile::CZipFile( const CString& strFilename )
    : m_dwCrc32(0),	m_dwCompressedSize(0),	m_dwFileOffset(0),
      m_compressionMethod(deflated)
{
    m_strFilename = GetZipCompatibleFilename( strFilename );

    CFileStatus filestatus;
    if ( CFile::GetStatus(strFilename, filestatus) )
	{
		m_fileDate.year = filestatus.m_mtime.GetYear()-1980;
		m_fileDate.month = filestatus.m_mtime.GetMonth();
		m_fileDate.day = filestatus.m_mtime.GetDay();

		m_fileTime.hour = filestatus.m_mtime.GetHour();
		m_fileTime.minute = filestatus.m_mtime.GetMinute();
		m_fileTime.twoSeconds = filestatus.m_mtime.GetSecond() / 2;

		m_bAttributes = filestatus.m_attribute;
		m_dwUncompressedSize = filestatus.m_size;
	}
}

CZipFile::CZipFile( const CZipFile& zf )
    : m_strFilename(zf.m_strFilename),    
	m_bAttributes(zf.m_bAttributes),
    m_fileTime(zf.m_fileTime), 
    m_fileDate(zf.m_fileDate),
    m_compressionMethod(zf.m_compressionMethod),
	m_dwCrc32(zf.m_dwCrc32),
	m_dwCompressedSize(zf.m_dwCompressedSize),
	m_dwUncompressedSize(zf.m_dwUncompressedSize),
	m_dwFileOffset(zf.m_dwFileOffset),
    m_strFileComment(zf.m_strFileComment)
{
}

CZipFile& CZipFile::operator=( const CZipFile& zf )
{
    m_strFilename = zf.m_strFilename; 
	m_bAttributes = zf.m_bAttributes;
    m_fileTime = zf.m_fileTime; 
	m_fileDate = zf.m_fileDate;
    m_compressionMethod = zf.m_compressionMethod;
	m_dwCrc32 = zf.m_dwCrc32;
	m_dwCompressedSize = zf.m_dwCompressedSize;
	m_dwUncompressedSize = zf.m_dwUncompressedSize;
	m_dwFileOffset = zf.m_dwFileOffset;
    m_strFileComment=  zf.m_strFileComment;

    return *this;
}

CString CZipFile::GetZipCompatibleFilename( const CString& strFilename )
{
    CString strZipFilename;

    // don't store drive letter...
    char szDirectory[_MAX_DIR];
    char szFilename[_MAX_FNAME];
    char szExtension[_MAX_EXT];
    _splitpath( strFilename, NULL, szDirectory, szFilename, szExtension );
    LPTSTR pszFilename = strZipFilename.GetBufferSetLength(MAX_PATH);
    _makepath( pszFilename, NULL, szDirectory, szFilename, szExtension );
    strZipFilename.ReleaseBuffer();
    // ... and replace all '\' with '/'
    strZipFilename.Replace( '\\', '/' );
    // remove leading
    if ( !strZipFilename.IsEmpty() && strZipFilename[0] == '/' )
        return strZipFilename.Right( strZipFilename.GetLength() - 1 );
    else
        return strZipFilename;
}


/////////////////////////////////////////////////////////////////////////////
// CZipArchive

CZipArchive::CZipArchive( const CString& strFilename, bool bCreateNew, CZipArchiveTarget * pTarget )
    : m_pTarget(pTarget), m_pCompressionBuf(NULL), m_pDecompressionBuf(NULL),
    m_bReadonly(false)
{
    m_pCompressionBuf = new unsigned char[COMPRESSION_BUFFER_SIZE];
    m_pDecompressionBuf = new unsigned char[DECOMPRESSION_BUFFER_SIZE];

    if ( bCreateNew )
    {
        m_fileZipArchive.Open( strFilename, CFile::modeCreate|CFile::modeReadWrite|CFile::shareDenyWrite );
            
        if ( !WriteCentralDirectory() )    // empty one
		{
            TRACE( "\nCZipArchive::CZipArchive : WriteCentralDirectory() failed\n" );
            AfxThrowUserException();
		}
    }
    else     // open existing file
    {
        DWORD dwFileAttributes = ::GetFileAttributes( strFilename );
        if ( dwFileAttributes == 0xffffffff )    // must exist
            AfxThrowUserException();
        // if this is readonly file, it must be open in readonly mode
        m_bReadonly = dwFileAttributes & FILE_ATTRIBUTE_READONLY? true : false;
        UINT nOpenFlags = CFile::shareDenyWrite | (m_bReadonly? CFile::modeRead : CFile::modeReadWrite);
        m_fileZipArchive.Open( strFilename, nOpenFlags );
        
        if ( !ReadCentralDirectory() )
        {
			TRACE( "\nCZipArchive::CZipArchive : ReadCentralDirectory() failed\n" );
            AfxThrowUserException();
        }
    }    
}

CZipArchive::~CZipArchive()
{
    if ( m_pCompressionBuf != NULL )
        delete[] m_pCompressionBuf;
    if ( m_pDecompressionBuf != NULL )
        delete[] m_pDecompressionBuf;
}

/////////////////////////////////////////////////////////////////////////////
// CZipArchive member functions

bool CZipArchive::WriteCentralDirectory()
{
    ASSERT(m_bReadonly==false);
    
    return WriteCentralDirectory( m_fileZipArchive, m_arrayZipFiles, m_strZipArchiveComment );
}

bool CZipArchive::WriteCentralDirectory( CFile& file, const CArray<CZipFile,CZipFile&>& arrayZipFiles, const CString& strZipArchiveComment )
{
	try
	{
		file.SeekToEnd();
        CENTRALDIRECTORYINFO centralDirInfo;
        ZeroMemory( &centralDirInfo, sizeof(centralDirInfo) );
		centralDirInfo.dwCentralDirOffset = file.GetPosition();
		centralDirInfo.dwCentralDirSize = 0;

		for ( int i = 0; i < arrayZipFiles.GetSize(); i++ )
		{
			const CZipFile& zipFile = arrayZipFiles[i];

			CENTRALFILEHEADER cfh;
			ZeroMemory( &cfh, sizeof(cfh) );
			cfh.dwSignature = CENTRALDIR_HEADER_SIGNATURE;
			cfh.versionMadeBy.cNumber = cfh.versionNeededToExtract.cNumber = MAKEVERSIONNUMBER(CURRENT_PKZIP_VERSION,0);
			cfh.versionMadeBy.cHostOS = cfh.versionNeededToExtract.cHostOS = OS_FAT;
			cfh.flags.deflateMethod = deflateMaximum;
			cfh.wCompressionMethod = deflated;
			cfh.fileTime = zipFile.GetFileTime();
			cfh.fileDate = zipFile.GetFileDate();
			cfh.dwCrc32 = zipFile.GetCrc32();
			cfh.dwCompressedSize = zipFile.GetCompressedSize();
			cfh.dwUncompressedSize = zipFile.GetUncompressedSize();
			cfh.wFilenameLength = zipFile.GetFilename().GetLength();
			cfh.wExtraFieldLength = 0;
			cfh.wFileCommentLength = zipFile.GetFileComment().GetLength();
			cfh.wLocatedOnDiskNo = 0;
			cfh.wInternalFileAttributes = 0;    // binary data
			cfh.dwExternalFileAttributes = zipFile.GetAttributes();
			cfh.dwLocalHeaderOffset = zipFile.GetLocalHeaderOffset();

			file.Write( &cfh, sizeof(cfh) );
			file.Write( (LPCTSTR) zipFile.GetFilename(), cfh.wFilenameLength );
			if ( cfh.wFileCommentLength > 0 )
				file.Write( (LPCTSTR) zipFile.GetFileComment(), cfh.wFileCommentLength );
    
			centralDirInfo.dwCentralDirSize += sizeof(cfh) + cfh.wFilenameLength + cfh.wFileCommentLength;
		}

		centralDirInfo.dwSignature = CENTRALDIR_INFO_SIGNATURE;
		centralDirInfo.wCentralDirFirstDisk = 0;
		centralDirInfo.wThisDiskNumber = 0;
		centralDirInfo.wTotalCentralDirEntriesCount = arrayZipFiles.GetSize();
		centralDirInfo.wThisDiskCentralDirEntriesCount = arrayZipFiles.GetSize();
		centralDirInfo.wZipFileCommentLength = strZipArchiveComment.GetLength();
		file.Write( &centralDirInfo, sizeof(centralDirInfo) );
		if ( strZipArchiveComment.GetLength() )
			file.Write( (LPCTSTR) strZipArchiveComment, strZipArchiveComment.GetLength() );

		return true;
	}
    catch ( CFileException * e )
    {
        e->Delete();
        TRACE( "\nCZipArchive::WriteCentralDirectory failed\n" );
        return false;
    }        
	catch ( ... )
	{
	    TRACE( "\nCZipArchive::WriteCentralDirectory failed\n" );
		return false;
	}
}

bool CZipArchive::ReadCentralDirectory()
{
	char * pTempBuf = new char [ MAX_ZIP_COMMENT_LENGTH + sizeof(CENTRALDIRECTORYINFO) ];
	
	try
	{
		int nFileOffset = m_fileZipArchive.SeekToEnd();

		if ( nFileOffset < sizeof(CENTRALDIRECTORYINFO) )
		{
			delete[] pTempBuf;
			return false;
		}

		nFileOffset -=  MAX_ZIP_COMMENT_LENGTH - sizeof(CENTRALDIRECTORYINFO) - 1;
		if ( nFileOffset < 0 ) nFileOffset = 0;

		m_fileZipArchive.Seek( nFileOffset, CFile::begin );
		UINT nBytesRead = m_fileZipArchive.Read( pTempBuf, m_fileZipArchive.GetLength() - nFileOffset - 1 );

		for ( int i = nBytesRead-sizeof(CENTRALDIRECTORYINFO)+1; i >= 0; i-- )
			if ( *((DWORD*)(pTempBuf+i)) == CENTRALDIR_INFO_SIGNATURE )
				break;

		delete[] pTempBuf;
		pTempBuf = NULL;

		if ( i < 0 )  return false;

		m_fileZipArchive.Seek( nFileOffset + i, CFile::begin );

        CENTRALDIRECTORYINFO centralDirInfo;
        ZeroMemory( &centralDirInfo, sizeof(centralDirInfo) );
		m_fileZipArchive.Read( &centralDirInfo, sizeof(centralDirInfo) );
		if ( centralDirInfo.dwSignature != CENTRALDIR_INFO_SIGNATURE )
			return false;
    
        DWORD dwEndOfCentralDirInfo = m_fileZipArchive.GetPosition();
		
		m_fileZipArchive.Seek( centralDirInfo.dwCentralDirOffset, CFile::begin );

		m_arrayZipFiles.RemoveAll();
		char szFilename[MAX_PATH];

		for ( int nFile = 0; nFile < centralDirInfo.wThisDiskCentralDirEntriesCount; nFile++ )
		{
			CENTRALFILEHEADER cfh;
			m_fileZipArchive.Read( &cfh, sizeof(cfh) );
			if ( cfh.dwSignature != CENTRALDIR_HEADER_SIGNATURE )
				return false;
			m_fileZipArchive.Read( szFilename, cfh.wFilenameLength );
			if ( cfh.wFilenameLength >= MAX_PATH )
				return false;
			ASSERT(cfh.wFilenameLength > 0);
			szFilename[cfh.wFilenameLength] = 0;

            CString strComment( _T("") );
			if ( cfh.wFileCommentLength > 0 )
			{
				LPTSTR p = strComment.GetBufferSetLength( cfh.wFileCommentLength );
				m_fileZipArchive.Read( p, cfh.wFileCommentLength );
				strComment.ReleaseBuffer();
			}

			m_arrayZipFiles.Add( CZipFile(cfh,szFilename,strComment) );
		}

        if ( centralDirInfo.wZipFileCommentLength > 0 )
        {
            m_fileZipArchive.Seek( dwEndOfCentralDirInfo, CFile::begin );
            LPTSTR p = m_strZipArchiveComment.GetBufferSetLength( centralDirInfo.wZipFileCommentLength );
            m_fileZipArchive.Read( p, centralDirInfo.wZipFileCommentLength );
            m_strZipArchiveComment.ReleaseBuffer();
        }            
        
		return true;
	}
    catch ( CFileException * e )
    {
		if ( pTempBuf != NULL )
			delete[] pTempBuf;
        e->Delete();
        TRACE( "\nCZipArchive::ReadCentralDirectory() failed\n" );
        return false;
    }        
	catch ( ... )
	{
		if ( pTempBuf != NULL )
			delete[] pTempBuf;
        TRACE( "\nCZipArchive::ReadCentralDirectory() failed\n" );
		return false;
	}
}


int CZipArchive::Extract( const CString& strFilename )
{
    CStringArray arrayFilesToExtract;
    arrayFilesToExtract.Add( strFilename );
    
    return Extract( arrayFilesToExtract );
}

int CZipArchive::Extract( const CStringArray& arrayFilesToExtract )
{
    int nSuccess = 0;
    for ( int nFile = 0; nFile < arrayFilesToExtract.GetSize(); nFile++ )
    {
    	for ( int i = 0; i < m_arrayZipFiles.GetSize(); i++ )
    	{
            if ( m_arrayZipFiles[i].GetFilename().CompareNoCase(arrayFilesToExtract[nFile]) == 0 )
    	    {
                if ( m_arrayZipFiles[i].GetAttributes() & FILE_ATTRIBUTE_DIRECTORY )    
                    nSuccess = CreatePath(m_arrayZipFiles[i].GetFilename())? 0 : -1;
                else
                {
                    switch ( m_arrayZipFiles[i].GetCompresionMethod() )
                    {
                        case deflated:
                            nSuccess = UncompressFile( m_arrayZipFiles[i] );    
                            break;
                        case stored:
                            nSuccess = RestoreFile( m_arrayZipFiles[i] );
                            break;
                        default:
                            ASSERT(false);    // compression method not supported
                            nSuccess =-1;
                            break;
                    }
                }
                    
            
                if ( nSuccess < 0 )  break;
            }
    	}
    }
    	
    return nSuccess;
}

int CZipArchive::UncompressFile( const CZipFile& zipFile )
{
	try
	{
        TCHAR szDir[_MAX_DIR];
        _splitpath( zipFile.GetFilename(), NULL, szDir, NULL, NULL );
        if ( szDir[0] != 0 && !CreatePath(szDir) )
            AfxThrowUserException();
            
		CFile fileToExtract( zipFile.GetFilename(), CFile::modeCreate|CFile::modeWrite|CFile::shareDenyWrite );

		m_fileZipArchive.Seek( zipFile.GetLocalHeaderOffset(), CFile::begin );
		LOCALFILEHEADER lfh;
		m_fileZipArchive.Read( &lfh, sizeof(lfh) );
		if ( lfh.dwSignature != LOCALFILE_HEADER_SIGNATURE )
			return -1;
        m_fileZipArchive.Seek( lfh.wFilenameLength + lfh.wExtraFieldLength, CFile::current );

		ZeroMemory( &m_zStream, sizeof(m_zStream) );

		if ( inflateInit2(&m_zStream, -MAX_WBITS) != Z_OK )
		{
			TRACE( "\nCZipArchive::UncompressFile : inflateInit2() failed\n" );
			return -1;
		}

		m_zStream.avail_in = 0;                             
		m_zStream.next_out = m_pDecompressionBuf;           
		m_zStream.avail_out = DECOMPRESSION_BUFFER_SIZE;    

		DWORD dwBytesReadSoFar = 0;
		DWORD dwBytesDecompressedSoFar = 0;
        DWORD dwBytesDecompressed = 0;
		DWORD dwCrc32 = 0;
		do
		{
			if ( m_zStream.avail_in == 0 ) 
			{
				m_zStream.avail_in = m_fileZipArchive.Read( m_pCompressionBuf, min((lfh.dwCompressedSize-dwBytesReadSoFar),COMPRESSION_BUFFER_SIZE) );
				dwBytesReadSoFar += m_zStream.avail_in;
				m_zStream.next_in = m_pCompressionBuf;  
			}

			int iRet = inflate( &m_zStream, Z_NO_FLUSH );
            if ( iRet == Z_DATA_ERROR )
                iRet = inflateSync( &m_zStream );
            if ( iRet < 0 )
            {
                inflateEnd( &m_zStream );
                return -1;
            }

            dwBytesDecompressed = DECOMPRESSION_BUFFER_SIZE - m_zStream.avail_out;     
			dwBytesDecompressedSoFar += dwBytesDecompressed;

			if ( dwBytesDecompressed > 0 )
			{
				fileToExtract.Write( m_pDecompressionBuf, dwBytesDecompressed );
				dwCrc32 = m_crc.GetCRC( m_pDecompressionBuf, dwBytesDecompressed, dwCrc32 );
    			m_zStream.next_out = m_pDecompressionBuf;
    			m_zStream.avail_out = DECOMPRESSION_BUFFER_SIZE;
			}

			if ( m_pTarget != NULL )
			{
				bool bContinue = m_pTarget->OnArchiveProgress( zipFile.GetFilename(), dwBytesReadSoFar, lfh.dwCompressedSize );
				if ( bContinue == false )
				{
				    inflateEnd( &m_zStream );
					return -1;
				}				    
			}
		}
        while ( dwBytesDecompressed > 0 );

		inflateEnd( &m_zStream );

		if ( dwCrc32 != lfh.dwCrc32 )
        {
		    fileToExtract.Close();
		    CFile::Remove( zipFile.GetFilename() );
		    return -1;
        }

		// date, time, attributes
		CFileStatus fileStatus;
		fileToExtract.GetStatus( fileStatus );
		CTime time( lfh.fileDate.year + 1980, lfh.fileDate.month, lfh.fileDate.day, lfh.fileTime.hour, lfh.fileTime.minute, lfh.fileTime.twoSeconds * 2 );
		fileStatus.m_ctime = fileStatus.m_atime = fileStatus.m_mtime = time;
		fileStatus.m_attribute = zipFile.GetAttributes();
		fileToExtract.Close();
		CFile::SetStatus( zipFile.GetFilename(), fileStatus );

		return 0;    // ok
	}
    catch ( CFileException * e )
    {
        e->Delete();
        return -1;
    }        
	catch ( ... )
	{
		return -1;
	}
}

int CZipArchive::RestoreFile( const CZipFile& zipFile )
{
    ASSERT(zipFile.GetCompresionMethod() == stored);

	try
	{
        TCHAR szDir[_MAX_DIR];
        _splitpath( zipFile.GetFilename(), NULL, szDir, NULL, NULL );
        if ( szDir[0] != 0 && !CreatePath(szDir) )
            AfxThrowUserException();
            
		CFile fileToRestore( zipFile.GetFilename(), CFile::modeCreate|CFile::modeWrite|CFile::shareDenyWrite );
		
		m_fileZipArchive.Seek( zipFile.GetLocalHeaderOffset(), CFile::begin );
		LOCALFILEHEADER lfh;
		m_fileZipArchive.Read( &lfh, sizeof(lfh) );
        m_fileZipArchive.Seek( lfh.wFilenameLength+lfh.wExtraFieldLength, CFile::current );

		DWORD dwBytesReadSoFar = 0;
		while ( dwBytesReadSoFar < lfh.dwCompressedSize )
		{
			DWORD dwBytesRead = m_fileZipArchive.Read( m_pCompressionBuf, min(lfh.dwCompressedSize-dwBytesReadSoFar,COMPRESSION_BUFFER_SIZE) );
			if ( dwBytesRead == 0 )  break;

			if ( m_pTarget != NULL )
			{
				bool bContinue = m_pTarget->OnArchiveProgress( zipFile.GetFilename(), dwBytesReadSoFar, lfh.dwCompressedSize );
				if ( bContinue == false )
				{
				    return -1;
				}
			}

			fileToRestore.Write( m_pCompressionBuf, dwBytesRead );
			dwBytesReadSoFar += dwBytesRead;
		}
		
		return dwBytesReadSoFar == lfh.dwCompressedSize? 0 : -1;
	}
    catch ( CFileException * e )
    {
        e->Delete();
        return -1;	
    }        
	catch ( ... )
	{
		return -1;
	}
}



int CZipArchive::Add( const CString& strFilename )
{
    CStringArray arrayFilesToAdd;
    arrayFilesToAdd.Add( strFilename );
    
    return Add( arrayFilesToAdd );
}

int CZipArchive::Add( const CStringArray& arrayFilesToAdd )
{
    if ( m_bReadonly )    return -1;    // archive opened as readonly
			
    // create temp file in the zip archive directory;
	char szTempFilename[_MAX_FNAME];
	::GetTempFileName( GetZipArchivePathOnly(), "~~~", 0, szTempFilename );
	CFile fileTemp( szTempFilename, CFile::modeCreate|CFile::modeReadWrite|CFile::shareDenyWrite );
       
    CArray<CZipFile,CZipFile&> arrayTempZipFiles;    
	for ( int i = 0; i < m_arrayZipFiles.GetSize(); i++ )
	{
        // Is this compressed file on the list of files to add?
        // If it is, skip its copying to temp file cause it will be compressed again
        bool bCopyToTempFile = true;
        for ( int nFile = 0; nFile < arrayFilesToAdd.GetSize(); nFile++ )
            if ( m_arrayZipFiles[i].GetFilename().CompareNoCase(arrayFilesToAdd[nFile]) == 0 )
            {
                bCopyToTempFile = false;
                break;
            }
        
		if ( bCopyToTempFile )
		{
	        DWORD dwStartingOffsetInNewFile = fileTemp.GetPosition();
			if ( !CopyZipFileTo(m_arrayZipFiles[i], fileTemp) )
			{
				fileTemp.Close();
				CFile::Remove( fileTemp.GetFilePath() );
				return -1;
			}
            int nPos = arrayTempZipFiles.Add( m_arrayZipFiles[i] );
            arrayTempZipFiles[nPos].SetLocalHeaderOffset( dwStartingOffsetInNewFile );
        }
    }
		
    // now compress files from the list
    for ( i = 0; i < arrayFilesToAdd.GetSize(); i++ )
    {
        CZipFile zipFile( arrayFilesToAdd[i] );			
        DWORD dwStartingOffsetInNewFile = fileTemp.GetPosition();
        if ( CompressFile(zipFile,fileTemp) < 0 )    
        {
            fileTemp.Close();
            CFile::Remove( fileTemp.GetFilePath() );
            return -1;
        }
        // compressed ok, add file to the list
        int nPos = arrayTempZipFiles.Add( zipFile );
        arrayTempZipFiles[nPos].SetLocalHeaderOffset( dwStartingOffsetInNewFile );
    }

    // write central directory into temp file, using temp compressed file list
    WriteCentralDirectory( fileTemp, arrayTempZipFiles, m_strZipArchiveComment );
    CString strFileName( fileTemp.GetFilePath() );
    fileTemp.Close();
    // replace old zip archive file with temp file (fileTemp -> m_fileArchive)
    return ReplaceArchiveFileWith( strFileName );    
}

bool CZipArchive::CopyZipFileTo( const CZipFile& zipFile, CFile& file )
{
	const int TRANSBUF_SIZE = 16384;
	unsigned char * pTransferBuf = new unsigned char [TRANSBUF_SIZE];
	
	try
	{
		m_fileZipArchive.Seek( zipFile.GetLocalHeaderOffset(), CFile::begin );
		LOCALFILEHEADER lfh;
		m_fileZipArchive.Read( &lfh, sizeof(lfh) );
		file.Write( &lfh, sizeof(lfh) );
		CString strFilename;
		LPTSTR p = strFilename.GetBufferSetLength( lfh.wFilenameLength );
		m_fileZipArchive.Read( p, lfh.wFilenameLength );
		strFilename.ReleaseBuffer();
		file.Write( strFilename, strFilename.GetLength() );

		DWORD dwBytesReadSoFar = 0;
		while ( dwBytesReadSoFar < lfh.dwCompressedSize )
		{
			DWORD dwBytesRead = m_fileZipArchive.Read( pTransferBuf, min(lfh.dwCompressedSize-dwBytesReadSoFar,TRANSBUF_SIZE) );
			if ( dwBytesRead == 0 )  break;

			if ( m_pTarget != NULL )
			{
				bool bContinue = m_pTarget->OnArchiveProgress( zipFile.GetFilename(), dwBytesReadSoFar, lfh.dwCompressedSize );
				if ( bContinue == false )
				{
					delete[] pTransferBuf;
					return false;
				}
			}

			file.Write( pTransferBuf, dwBytesRead );
			dwBytesReadSoFar += dwBytesRead;
		}
		
		delete[] pTransferBuf;

		return dwBytesReadSoFar == lfh.dwCompressedSize? true : false;
	}
    catch ( CFileException * e )
    {
        e->Delete();
        if ( pTransferBuf != NULL )
            delete[] pTransferBuf;
        return false;	
    }        
	catch ( ... )
	{
        if ( pTransferBuf != NULL )
            delete[] pTransferBuf;
		return false;
	}
}

int CZipArchive::CompressFile( CZipFile& zipFile, CFile& file )
{
    ASSERT(m_bReadonly==false);
    
	try
	{
		CFile fileToCompress( zipFile.GetFilename(), CFile::modeRead|CFile::shareDenyWrite );

		
		DWORD dwLocalHeaderPos = file.GetPosition();

		LOCALFILEHEADER lfh;
		ZeroMemory( &lfh, sizeof(lfh) );

		lfh.dwSignature = LOCALFILE_HEADER_SIGNATURE;
		lfh.versionNeededToExtract.cHostOS = OS_FAT;
		lfh.versionNeededToExtract.cNumber = MAKEVERSIONNUMBER(CURRENT_PKZIP_VERSION,0);
		lfh.flags.deflateMethod = deflateMaximum;
		lfh.wCompressionMethod = deflated;
		CFileStatus filestatus;
		fileToCompress.GetStatus( filestatus );
		lfh.fileTime.hour = filestatus.m_mtime.GetHour();
		lfh.fileTime.minute = filestatus.m_mtime.GetMinute();
		lfh.fileTime.twoSeconds = filestatus.m_mtime.GetSecond() / 2;
		lfh.fileDate.year = filestatus.m_mtime.GetYear()-1980;
		lfh.fileDate.month = filestatus.m_mtime.GetMonth();
		lfh.fileDate.day = filestatus.m_mtime.GetDay();
		lfh.dwUncompressedSize = filestatus.m_size;
		lfh.wFilenameLength = zipFile.GetFilename().GetLength();
      
		file.Write( &lfh, sizeof(lfh) );
		file.Write( zipFile.GetFilename(), lfh.wFilenameLength );

		ZeroMemory( &m_zStream, sizeof(m_zStream) );
		int nInitCode = deflateInit2( &m_zStream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, -MAX_WBITS, DEF_MEM_LEVEL, Z_DEFAULT_STRATEGY );
		if ( nInitCode != Z_OK )
		{
			TRACE( "\nCZipArchive::CompressFile : deflateInit2() failed\n" );
			return -1;
		}

		m_zStream.avail_in = 0;                             
		m_zStream.next_out = m_pCompressionBuf;           
		m_zStream.avail_out = COMPRESSION_BUFFER_SIZE;    

		DWORD dwBytesReadSoFar = 0;
		DWORD dwBytesCompressedSoFar = 0;
		DWORD dwCrc32 = 0;
		while ( dwBytesReadSoFar < lfh.dwUncompressedSize ) 
		{
			m_zStream.avail_in = fileToCompress.Read( m_pDecompressionBuf, min(lfh.dwUncompressedSize-dwBytesReadSoFar,DECOMPRESSION_BUFFER_SIZE) );
			dwBytesReadSoFar += m_zStream.avail_in;
			m_zStream.next_in = m_pDecompressionBuf;  

			dwCrc32 = m_crc.GetCRC( m_pDecompressionBuf, m_zStream.avail_in, dwCrc32 );

			while ( m_zStream.avail_in != 0 ) 
			{
				if ( m_zStream.avail_out == 0 ) 
				{
					file.Write( m_pCompressionBuf, COMPRESSION_BUFFER_SIZE );	
					m_zStream.next_out = m_pCompressionBuf;
					m_zStream.avail_out = COMPRESSION_BUFFER_SIZE;
				}
				int iRet = deflate( &m_zStream, Z_NO_FLUSH );
				if ( iRet != Z_OK ) break;
			}

			if ( m_pTarget != NULL )
			{
				bool bContinue = m_pTarget->OnArchiveProgress( zipFile.GetFilename(), dwBytesReadSoFar, lfh.dwUncompressedSize );
				if ( bContinue == false )
				{
                    deflateEnd( &m_zStream );
				    return -1;
				}
			}
		}

		bool bDone = false;
		for (;;) 
		{
			DWORD dwBytesCompressed = COMPRESSION_BUFFER_SIZE - m_zStream.avail_out;

			if ( dwBytesCompressed > 0 ) 
			{
				file.Write( m_pCompressionBuf, dwBytesCompressed );
				m_zStream.next_out = m_pCompressionBuf;
				m_zStream.avail_out = COMPRESSION_BUFFER_SIZE;
			}
        
			if ( bDone ) break;

			int iRetCode = deflate( &m_zStream, Z_FINISH );

			// Ignore the second of two consecutive flushes:
			if ( dwBytesCompressed == 0 && iRetCode == Z_BUF_ERROR ) 
				iRetCode = Z_OK;

			// deflate has finished flushing only when it hasn't used up
			// all the available space in the output buffer: 
			bDone = m_zStream.avail_out != 0 || iRetCode == Z_STREAM_END;
 
			if ( iRetCode != Z_OK && iRetCode != Z_STREAM_END ) 
				break;
		}

		file.Seek( dwLocalHeaderPos, CFile::begin );
		zipFile.SetCrc32( dwCrc32 );
		zipFile.SetCompressedSize( m_zStream.total_out );
		zipFile.SetLocalHeaderOffset( dwLocalHeaderPos );

		lfh.dwCrc32 = dwCrc32;
		lfh.dwCompressedSize = m_zStream.total_out;
		file.Write( &lfh, sizeof(lfh) );    

		file.SeekToEnd();

		deflateEnd( &m_zStream );

		return 0;	// ok
	}
    catch ( CFileException * e )
    {
        e->Delete();
        return -1;	
    }        
	catch ( ... )
	{
		return -1;
	}
}

CString CZipArchive::GetZipArchivePathOnly() const
{
    char szDrive[_MAX_DRIVE];
    char szDir[_MAX_DIR];
    _splitpath( m_fileZipArchive.GetFilePath(), szDrive, szDir, NULL, NULL );
    
    CString strZipArchivePath;
    LPSTR pszPath = strZipArchivePath.GetBufferSetLength( MAX_PATH );
    _makepath( pszPath, szDrive, szDir, NULL, NULL );
    strZipArchivePath.ReleaseBuffer();
    
    return strZipArchivePath;
}

int CZipArchive::ReplaceArchiveFileWith( const CString& strFile )
{
    try
	{
        CString strArchiveName( m_fileZipArchive.GetFilePath() );
        m_fileZipArchive.Close();
    	CFile::Remove( strArchiveName );
    	if ( ::MoveFile(strFile, strArchiveName) )
    	{
            m_fileZipArchive.Open( strArchiveName, CFile::modeReadWrite|CFile::shareDenyWrite );
            return ReadCentralDirectory()? 0 : -1;
    	}    	    
        else
    	    return -1;
    }
    catch ( CFileException * e )
    {
        e->Delete();
        return -1;
    }
    catch ( ... )
    {
        return -1;
    }
}


int CZipArchive::Delete( const CString& strFile )
{
    CStringArray arrayFilesToDelete;
    arrayFilesToDelete.Add( strFile );
    
    return Delete( arrayFilesToDelete );
}    

int CZipArchive::Delete( const CStringArray& arrayFilesToDelete )
{
    char szTempFilename[MAX_PATH];
	::GetTempFileName( GetZipArchivePathOnly(), "~~~", 0, szTempFilename );
	CFile fileTemp( szTempFilename, CFile::modeCreate|CFile::modeReadWrite|CFile::shareDenyWrite );
        
    CArray<CZipFile,CZipFile&> arrayTempZipFiles;    

	for ( int i = 0; i < m_arrayZipFiles.GetSize(); i++ )
	{
        bool bCopyToTempFile = true;
        for ( int nFile = 0; nFile < arrayFilesToDelete.GetSize(); nFile++ )
            if ( m_arrayZipFiles[i].GetFilename().CompareNoCase(arrayFilesToDelete[nFile]) == 0 )
            {
                bCopyToTempFile = false;
                break;
            }
        
		if ( bCopyToTempFile )
		{
            DWORD dwStartingOffsetInNewFile = fileTemp.GetPosition();
		    if ( !CopyZipFileTo(m_arrayZipFiles[i], fileTemp) )
            {
                fileTemp.Close();
                CFile::Remove( fileTemp.GetFilePath() );
                return -1;
            }						
            
            int nPos = arrayTempZipFiles.Add( m_arrayZipFiles[i] );
            arrayTempZipFiles[nPos].SetLocalHeaderOffset( dwStartingOffsetInNewFile );
        }
    }
    
    WriteCentralDirectory( fileTemp, arrayTempZipFiles, m_strZipArchiveComment );
    CString strFileName( fileTemp.GetFilePath() );
    fileTemp.Close();

    return ReplaceArchiveFileWith( strFileName );    // temp -> m_fileArchive
}

bool CZipArchive::CreatePath( const CString& strPath, int nCurrPos /* 0 */ )
{
    VERIFY(nCurrPos < MAX_PATH);

    TCHAR szCurrentDirectory[MAX_PATH];
    ::GetCurrentDirectory( MAX_PATH, szCurrentDirectory );

    bool bSuccess = false;
    CString strDirectoryToCreate;
    strDirectoryToCreate = strPath.Mid( nCurrPos );
    strDirectoryToCreate = strDirectoryToCreate.SpanExcluding( "/" );

    bSuccess = ::SetCurrentDirectory(strDirectoryToCreate)? true : false;
    if ( !bSuccess )
    {
        bSuccess = ::CreateDirectory(strDirectoryToCreate, NULL)? true : false;
        if ( bSuccess )
        {
            nCurrPos += strDirectoryToCreate.GetLength() + 1;
            if ( nCurrPos < strPath.GetLength() )
            {
                ::SetCurrentDirectory( strDirectoryToCreate );
                bSuccess = CreatePath( strPath, nCurrPos );
                if ( !bSuccess )
                    ::RemoveDirectory( strDirectoryToCreate );
            }
        }
    }
    else
    {
        nCurrPos += strDirectoryToCreate.GetLength() + 1;
        if ( nCurrPos < strPath.GetLength() )
            bSuccess = CreatePath( strPath, nCurrPos );
    }
                
    ::SetCurrentDirectory( szCurrentDirectory );
    return bSuccess;
}   

