//////////////////////////////////////
// Vladimir Klisic (klisa@sezampro.yu)
// PC #45, April 1999.

#if !defined(AFX_ZIPARCHIVE_H__8DFA65B1_B0AC_11D2_8FBD_000000000000__INCLUDED_)
#define AFX_ZIPARCHIVE_H__8DFA65B1_B0AC_11D2_8FBD_000000000000__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// ZipArchive.h : header file
//

#include <afxtempl.h>
#include "zlib.h"
#include "pkzip.h"
#include "crc32.h"

/////////////////////////////////////////////////////////////////////////////
// CZipArchive command target
class CZipArchiveTarget
{
public:
    virtual bool OnArchiveProgress( const CString& strFilename, DWORD dwCurrent, DWORD dwTotal ) = 0;
};

class CZipFile
{
protected:
    CString m_strFilename;    
	BYTE m_bAttributes;        
    DOSTIME m_fileTime;				            
	DOSDATE m_fileDate;				            
    enumCompressionMethod m_compressionMethod;
	DWORD m_dwCrc32;                
	DWORD m_dwCompressedSize;       
	DWORD m_dwUncompressedSize;     
	DWORD m_dwFileOffset;                       
    CString m_strFileComment;
    
    CString GetZipCompatibleFilename( const CString& strFilename );
    
public:
    CZipFile( const CString& strFilename );
    CZipFile( const CENTRALFILEHEADER& cfh, const CString& strFilename, LPCTSTR pszFileComment=NULL );
    CZipFile( const CZipFile& zf );
    CZipFile& operator=( const CZipFile& zf );
    CZipFile() {};

    const CString& GetFilename() const { return m_strFilename; }
    const CString& GetFileComment() const { return m_strFileComment; }
    const DOSTIME& GetFileTime() const { return m_fileTime; }
    const DOSDATE& GetFileDate() const { return m_fileDate; }
    enumCompressionMethod GetCompresionMethod() const { return m_compressionMethod; }
    DWORD GetCrc32() const { return m_dwCrc32; }
    DWORD GetCompressedSize() const { return m_dwCompressedSize; }
    DWORD GetUncompressedSize() const { return m_dwUncompressedSize; }
    BYTE GetAttributes() const { return m_bAttributes; }
    DWORD GetLocalHeaderOffset() const { return m_dwFileOffset; }

    void SetLocalHeaderOffset( DWORD dwOffset ) { m_dwFileOffset = dwOffset; }
    void SetCompressedSize( DWORD dwSize )  { m_dwCompressedSize = dwSize; }
    void SetUncompressedSize( DWORD dwSize )  { m_dwUncompressedSize = dwSize; }
    void SetCrc32( DWORD dwCrc32 ) { m_dwCrc32 = dwCrc32; }
};

const int COMPRESSION_BUFFER_SIZE = 16384;
// zlib.h: avail_out must be at least 0.1% larger than avail_in plus 12 bytes
const int DECOMPRESSION_BUFFER_SIZE = COMPRESSION_BUFFER_SIZE + 2048;

class CZipArchive 
{
public:
	CZipArchive( const CString& strFilename, bool bCreateNew, CZipArchiveTarget * pTarget = NULL );
	virtual ~CZipArchive();

    CString GetName() const { return m_fileZipArchive.GetFileName(); }
    bool CanAddFiles() const { return m_bReadonly == false; }
    int Add( const CString& strFilename );
    int Add( const CStringArray& arrayFilesToAdd );
    int Extract( const CString& strFile );
    int Extract( const CStringArray& arrayFilesToExtract );
    int Delete( const CString& strFile );
    int Delete( const CStringArray& arrayFilesToDelete );
    int GetZipFileCount() const { return m_arrayZipFiles.GetSize(); }
    CZipFile GetZipFile( int nPos ) const { return m_arrayZipFiles[nPos]; }
    CZipFile& GetZipFile( int nPos ) { return m_arrayZipFiles[nPos]; }

protected:
    CFile m_fileZipArchive;
    bool m_bReadonly;    
    CString m_strZipArchiveComment;
    CArray<CZipFile,CZipFile&> m_arrayZipFiles;    
    CZipArchiveTarget * m_pTarget;
    CCrc32 m_crc;
    z_stream m_zStream;
    unsigned char * m_pCompressionBuf;
    unsigned char * m_pDecompressionBuf;

    int CompressFile( CZipFile& zipFile, CFile& file );    
    int UncompressFile( const CZipFile& zipFile );    
    int RestoreFile( const CZipFile& zipFile );
    bool CopyZipFileTo( const CZipFile& zipFile, CFile& file );
    int ReplaceArchiveFileWith( const CString& strFile );
    bool CreatePath( const CString& strPath, int nCurrPos=0 );    
    CString GetZipArchivePathOnly() const;
    
    bool ReadCentralDirectory();
    bool WriteCentralDirectory();
    bool WriteCentralDirectory( CFile& file, const CArray<CZipFile,CZipFile&>& arrayZipFiles, const CString& strZipArchiveComment );
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_ZIPARCHIVE_H__8DFA65B1_B0AC_11D2_8FBD_000000000000__INCLUDED_)
